## {{texts.common_parameters}}

{{texts.common_parameters_about}}
{%- for name, param in components.parameters.items() %}

### {{name}}

{% with rows = [[texts.name, texts.parameter_location, texts.type, texts.default, texts.nullable, texts.description]] %}
{%- for param in parameters -%}
{%- set _ = rows.append([param.name, param.in, read_dict(param, "schema", "type"), read_dict(param, "schema", "default", default=""), texts.get_yes_no(read_dict(param, "schema", "nullable", default=False)), read_dict(param, "description", default="")]) -%}
{%- endfor -%}
{{ rows | table }}
{% endwith -%}
{% endfor -%}
