# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: microservice.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 5, 28, 1, "", "microservice.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12microservice.proto\x12\x02ms"\x1b\n\nSubRequest\x12\r\n\x05topic\x18\x01 \x01(\t"\x1d\n\x0cUnsubRequest\x12\r\n\x05topic\x18\x01 \x01(\t"*\n\x0b\x44\x61taRequest\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t"+\n\x0c\x44\x61taResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t"\x07\n\x05\x45mpty2\x8b\x01\n\x04Grpc\x12/\n\tSubscribe\x12\x0e.ms.SubRequest\x1a\x10.ms.DataResponse0\x01\x12*\n\x0bUnsubscribe\x12\x10.ms.UnsubRequest\x1a\t.ms.Empty\x12&\n\x08SendData\x12\x0f.ms.DataRequest\x1a\t.ms.Emptyb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "microservice_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_SUBREQUEST"]._serialized_start = 26
    _globals["_SUBREQUEST"]._serialized_end = 53
    _globals["_UNSUBREQUEST"]._serialized_start = 55
    _globals["_UNSUBREQUEST"]._serialized_end = 84
    _globals["_DATAREQUEST"]._serialized_start = 86
    _globals["_DATAREQUEST"]._serialized_end = 128
    _globals["_DATARESPONSE"]._serialized_start = 130
    _globals["_DATARESPONSE"]._serialized_end = 173
    _globals["_EMPTY"]._serialized_start = 175
    _globals["_EMPTY"]._serialized_end = 182
    _globals["_GRPC"]._serialized_start = 185
    _globals["_GRPC"]._serialized_end = 324
# @@protoc_insertion_point(module_scope)
