class ModuleMetadata:
    Imports: str = "imports"
    Exports: str = "exports"
    Providers: str = "providers"
    Controllers: str = "controllers"
    Global: str = "_global_"
    Root: str = "_root_"
    Module: str = "_is_module_"
